#lang racket

(require plot)

;; by kent dybvig (TSPL), adapted from Sam M. Daniel (1984)
;; URL : http://www.scheme.com/tspl3/examples.html#./examples:h9
;; given a list of length 2^n, compute the FFT of it (resulting values are complex)
(define (dft x)
 (define (w-powers n)
   (let ((pi (* (acos 0.0) 2)))
     (let ((delta (/ (* -2.0i pi) n)))
       (let f ((n n) (x 0.0))
         (if (= n 0)
             '()
             (cons (exp x) (f (- n 2) (+ x delta))))))))
 (define (evens w)
   (if (null? w)
       '()
       (cons (car w) (evens (cddr w)))))
 (define (interlace x y)
   (if (null? x)
       '()
       (cons (car x) (cons (car y) (interlace (cdr x) (cdr y))))))
 (define (split ls)
   (let split ((fast ls) (slow ls))
     (if (null? fast)
         (values '() slow)
         (call-with-values
           (lambda () (split (cddr fast) (cdr slow)))
           (lambda (front back)
             (values (cons (car slow) front) back))))))
 (define (butterfly x w)
   (call-with-values
     (lambda () (split x))
     (lambda (front back)
       (values
         (map + front back)
         (map * (map - front back) w)))))
 (define (rfft x w)
   (if (null? (cddr x))
       (let ((x0 (car x)) (x1 (cadr x)))
         (list (+ x0 x1) (- x0 x1)))
       (call-with-values
         (lambda () (butterfly x w))
         (lambda (front back)
           (let ((w (evens w)))
             (interlace (rfft front w) (rfft back w)))))))
 (rfft x (w-powers (length x))))

#;(define (mag x)
 (sqrt (+ (* (re x) (re x)) (* (im x) (im x)))))

(define data (build-list 4096 (lambda (t) (+ (sin (* t (/ (* 2 pi) 5))) 0.0))))
(define dft-points (map magnitude (dft data)))
(define dft-max (apply max dft-points))
(define d (for/list ([j (in-list dft-points)]
                    [i (in-naturals)])
           (vector i j)))

(plot (points d)
     #:x-min -100
     #:x-max 5000
     #:y-max (* 1.2 dft-max)
     #:width 1100)
